"""
Drive-casa is an interfacing package for scripting of CASA from a separate
Python process (see :ref:`introduction`).

The package includes several convenience routines that allow chaining of CASA
commands, see :py:mod:`.commands` module.
"""

from __future__ import absolute_import
import os
import sys

from pkg_resources import get_distribution, DistributionNotFound

import drivecasa.commands
import drivecasa.utils
from drivecasa.casa_env import casapy_env
from drivecasa.interface import Casapy

# Can hard-code a default casa-dir here for easy running of unit-tests,
# if required:
default_casa_dir = None  # (If available already via external environment paths)
# default_casa_dir = '/opt/soft/builds/casapy-active'

default_test_ouput_dir = '/tmp/drivecasa-tests'


###########################################################
# Versioning; see also
# http://stackoverflow.com/questions/17583443
###########################################################
try:
    _dist = get_distribution('drive-casa')
    #The version number according to Pip:
    _nominal_version = _dist.version
    if not __file__.startswith(os.path.join(_dist.location, 'drivecasa')):
        # not installed, but there is another version that *is*
        raise DistributionNotFound
except DistributionNotFound:
    #The actual copy in use if a custom PYTHONPATH or local dir import is used
    __version__ = 'Local import @ '+os.path.dirname(os.path.abspath(__file__))
else:
    __version__ = _nominal_version


#!/usr/bin/env python

from setuptools import setup

setup(
    name="drive-casa",
    version="0.7.1",
    packages=['drivecasa'],
    description="""An interfacing package for scripting CASA from within a larger pipeline.""",
    author="Tim Staley",
    author_email="timstaley337@gmail.com",
    url="https://github.com/timstaley/drive-casa",
    license='BSD 3-clause',
    install_requires=['pexpect>4'],
)



import os
import collections

def ensure_dir(dirname):
    """
    Ensure directory exists.

    Roughly equivalent to `mkdir -p`
    """
    if not os.path.isdir(dirname):
        os.makedirs(dirname)

def derive_out_path(in_paths, out_dir, out_extension='',
                    strip_in_extension=True,
                    out_prefix=None):
    """
    Derives an 'output' path given some 'input' paths and an output directory.

    In the simple case that only a single path is supplied, this is
    simply the pathname resulting from replacing extension suffix and moving
    dir, e.g.

    ``input_dir/basename.in`` -> ``output_dir/basename.out``

    If the out_dir is specified as 'None' then it is assumed that the
    new file should be located in the same directory as the first
    input path.

    In the case that multiple input paths are supplied, their basenames
    are concatenated, e.g.

        ``in_dir/base1.in`` + ``in_dir/base2.in``
            -> ``out_dir/base1_base2.out``

    If the resulting output path is identical to any input path, this
    raises an exception.

    NB the extension should be supplied including the '.' prefix.
    """
    in_paths = listify(in_paths)
    if out_dir is None:
        out_dir = os.path.dirname(in_paths[0])

    in_basenames = [os.path.basename(ip) for ip in in_paths]
    if strip_in_extension:
        in_basenames = [ os.path.splitext(bn)[0] for bn in in_basenames ]


    out_basename = '_'.join(in_basenames)
    if out_prefix:
        out_basename = out_prefix+out_basename
    out_path = os.path.join(out_dir, out_basename + out_extension)
    for ip in in_paths:
        if os.path.abspath(out_path) == os.path.abspath(ip):
            raise RuntimeError(
               'Specified path derivation results in output overwriting input!')
    return out_path

def save_script(script, filename):
    """Save a list of casa commands as a text file"""
    with open(filename, 'w') as fp:
        fp.write('\n'.join(script))

def get_circular_mask_string(centre_ra_dec_posns, aperture_radius="1arcmin"):
    """Get a mask string representing circular apertures about (x,y) tuples"""
    mask = ''
    if centre_ra_dec_posns is None:
        return mask
    for coords in centre_ra_dec_posns:
        mask += 'circle [ [ {x} , {y}] , {r} ]\n'.format(
                          x=coords[0], y=coords[1], r=aperture_radius)
    return mask

def get_box_mask_string(centre_pix_posns, width):
    """Get a mask string representing box apertures about (x,y) tuples"""
    mask = ''
    if centre_pix_posns is None:
        return mask
    for coords in centre_pix_posns:
        mask += 'box [ [{lx}pix , {ly}pix]  , [{hx}pix, {hy}pix] ]\n'.format(
                          lx=coords[0] - width / 2, ly=coords[1] - width / 2,
                          hx=coords[0] + width / 2, hy=coords[1] + width / 2,)
    return mask

def byteify(input):
    """
    Co-erce unicode to 'bytestring'

    (or string containing unicode, or dict containing unicode)
    Useful when e.g. importing filenames from JSON
    (CASA sometimes breaks if passed Unicode strings.)

    cf http://stackoverflow.com/a/13105359/725650
    """
    if isinstance(input, dict):
        return {byteify(key):byteify(value) for key,value in input.iteritems()}
    elif isinstance(input, list):
        return [byteify(element) for element in input]
    elif isinstance(input, unicode):
        return input.encode('utf-8')
    else:
        return input

def listify(x):
    """
    Ensure x is a (non-string) iterable; if not, enclose in a list.

    Returns:
        x or [x], accordingly.
    """
    if isinstance(x, basestring):
        return [x]
    elif isinstance(x, collections.Iterable):
        return x
    else:
        return [x]